'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const voidjsApp = 'VOID_JS_APP';

(function (ComponentError) {
  ComponentError["functionComponentOnly"] = "Only Function Component is supported";
  ComponentError["namedFunctionComponentOnly"] = "Anonymous Function Component is not supported";
  ComponentError["reservedFunctionName"] = "Reserved Component Name";
})(exports.ComponentError || (exports.ComponentError = {}));

function index (babel) {
  const template = babel.template; // store the declaration name of default export

  let exportDefaultDeclarationName = '';
  const defaultVisitor = {
    ExportDefaultDeclaration: {
      enter(path) {
        if (path.node.declaration.type !== 'FunctionDeclaration') {
          if (path.node.declaration.type === 'Identifier') {
            if (path.scope.hasBinding(path.node.declaration.name)) {
              var _path$scope$getBindin;

              if (((_path$scope$getBindin = path.scope.getBinding(path.node.declaration.name)) === null || _path$scope$getBindin === void 0 ? void 0 : _path$scope$getBindin.path.node.type) === 'FunctionDeclaration') {
                exportDefaultDeclarationName = path.node.declaration.name;
              } else {
                // that Identifier is not defined as FunctionDeclaration
                // should throw
                throw new Error(exports.ComponentError.functionComponentOnly);
              }
            } else {
              throw new Error(exports.ComponentError.functionComponentOnly);
            }
          } else {
            throw new Error(exports.ComponentError.functionComponentOnly);
          }
        }

        if (path.node.declaration.type === 'FunctionDeclaration') {
          if (!path.node.declaration.id) throw new Error(exports.ComponentError.namedFunctionComponentOnly);
          if (path.node.declaration.id.name === voidjsApp) throw new Error(exports.ComponentError.reservedFunctionName);
          exportDefaultDeclarationName = path.node.declaration.id.name;
        }
      }

    }
  };
  return {
    visitor: {
      Program: {
        enter(path) {
          if (!path.scope.hasBinding('createElement')) {
            path.unshiftContainer('body', template.ast(`import { createElement } from "react";`));
          }

          if (!path.scope.hasBinding('ReactDOM')) {
            path.unshiftContainer('body', template.ast(`import ReactDOM from "react-dom";`));
          }

          path.traverse(defaultVisitor);
        },

        exit(path, state) {
          // no named default export found
          if (exportDefaultDeclarationName === '') return;
          const {
            opts: {
              root = 'app',
              hydrate = false,
              app = false
            }
          } = state;
          const h = `ReactDOM.${hydrate ? 'hydrate' : 'render'}`;
          const container = `document.getElementById("${root}")`;
          path.pushContainer('body', template.ast(`const __frontmatter__ = typeof frontmatter !== 'undefined' ? frontmatter : {}; if (typeof getStaticProps !== "undefined") {
                (async function () {
                  const data = await getStaticProps();
                  const pageProps = Object.assign(__frontmatter__, data.props)
                  if (${!!app}) {
                    import("${app}").then(({default: ${voidjsApp}}) => {
                      ${h}(createElement(${voidjsApp}, {Component: ${exportDefaultDeclarationName}, pageProps: pageProps}), ${container});
                    });
                  } else {
                    ${h}(createElement(${exportDefaultDeclarationName}, pageProps), ${container});
                  }
                })();
              } else {
                if (${!!app}) {
                  import("${app}").then(({default: ${voidjsApp}}) => {
                    ${h}(createElement(${voidjsApp}, {Component: ${exportDefaultDeclarationName}, pageProps: __frontmatter__}), ${container});
                  });
                } else {
                  ${h}(createElement(${exportDefaultDeclarationName}, __frontmatter__), ${container});
                }
              }`));
        }

      }
    }
  };
}

exports.default = index;
